*****
* Name          strucpy.prg
* Date          28 dec 90
* Last change   12 sep 92
* Release       2.0
* Notice        Copyright 1990-1992 C.L.E. bvba
* Compile       Clipper strucopy /m/n/a/w    (clipper version 5.01)
* link          rtlink fi strucopy (or other linker)
* Note          utility to update database structures
*****

*
* This is a handy utility I use for some time now.
* I decided to place this code in public domain, so others can
* benefit from it.
* Feel free to use and modify it, If you have any suggestions
* or ideas, please let me know...
*
* Dirk Sommerlinck (CIS ID: 100022,1205)
*

#include "directry.ch"
#include "dbstruct.ch"

Static NumOfRec,RecCounter

Procedure strucopy (sourcepath,destpath,extension)

  local srcarray
  set exact on

  ?
  ? "Structure copy utility for dBase files - Version 2.0 (Public domain)"
  ? "Copyright (c) 1990 - 1992 C.L.E. bvba - BELGIUM"
  ? "Author: Dirk Sommerlinck (CIS ID: 100022,1205)"
  ?

  if sourcepath=NIL
    disphelp()
  else
  
    * Check command line parameters

    if destpath=NIL
      destpath="\"+curdir()
    elseif left(destpath,1)="."
      extension=destpath
      destpath="\"+curdir()
    endif
    
    if extension=NIL
      extension=".DBF"
    endif
    
    sourcepath=alltrim(sourcepath)
    if !right(sourcepath,1)$":\"
      sourcepath=sourcepath+"\"
    endif
  
    destpath=alltrim(destpath)
    if !right(destpath,1)$":\"
      destpath=destpath+"\"
    endif
    
    * Do the actual stuff
    
    ? "Source directory: "+sourcepath
    ?
    
    srcarray=directory(sourcepath+"*"+extension)
    aeval(srcarray,{|afile| do_it(sourcepath,destpath,afile[F_NAME])})
    
    ?
    ?
    
  endif
    
return
  

function do_it (src,dest,dbffile)
  local newstruct
  local oldfile   :="$STRUCPY.DBF"
  local olddbtfile:="$STRUCPY.DBT"
  local dbtfile   :=left(dbffile,at(".",dbffile)-1)+".DBT"
  
  * ERASE OLD FILES IF THEY EXIST
  
  erase (dest+oldfile)
  erase (dest+olddbtfile)
  
  * PERFORM COMPARISON AND ACT ACCORINGLY

  if different(src+dbffile,dest+dbffile,@NumOfRec)
  
    * SAVE EXISTING DATA IF THE FILES EXIST
    
    rename (dest+dbffile) to (dest+oldfile)
    rename (dest+dbtfile) to (dest+olddbtfile)
    
    * GET NEW STRUCTURE
    
    use (src+dbffile)
    newstruct=dbstruct()
    use
    
    * CREATE NEW STRUCTURE
    
    dbcreate(dest+dbffile,newstruct)
    use (dest+dbffile)

    if file(dest+oldfile)

      * RESTORE EXISTING DATA
      
      ? "Modify structure  : "+dest+dbffile+" -> "
      
      append from (dest+oldfile) for DispBusy()
      use
      
      * DELETE OLD STRUCTURE (AND MEMO FILE IF IT EXISTS)
      
      erase (dest+oldfile)
      erase (dest+olddbtfile)
      
      ?? " OK "
      
    else
    
      use
      ? "New structure     : "+dest+dbffile
      
    endif
    
  else
  
    ? "Same structure    : "+dest+dbffile
    
  endif
  
return nil


function different (srcfile,destfile)
  local src_struc,dest_struc,src_flds,dest_flds,atfield:=1,isdif:=.T.
  if file(destfile) // STILL THE SAME
    use (srcfile)
    src_struc=asort(dbstruct(),,,{|x,y| x[DBS_NAME]<y[DBS_NAME]})
    use (destfile)
    NumOfRec=iif(Lastrec()=0,1,Lastrec())
    RecCounter=0
    dest_struc=asort(dbstruct(),,,{|x,y| x[DBS_NAME]<y[DBS_NAME]})
    use
    src_flds=len(src_struc)
    dest_flds=len(dest_struc)
    if src_flds=dest_flds // STILL THE SAME
      do while atfield<=src_flds
        if !(src_struc[atfield,DBS_NAME]==dest_struc[atfield,DBS_NAME]) .or. ;
           !(src_struc[atfield,DBS_TYPE]==dest_struc[atfield,DBS_TYPE]) .or. ;
             src_struc[atfield,DBS_LEN ]<>dest_struc[atfield,DBS_LEN ]  .or. ;
             src_struc[atfield,DBS_DEC ]<>dest_struc[atfield,DBS_DEC ]
                atfield=src_flds+10
        else
          ++atfield
        endif
      enddo
      isdif=(atfield==src_flds+10)
    endif
  endif
return isdif

procedure disphelp  
  ? "Syntax: STRUCOPY Source [destination] [extension]"
  ?
  ? "Strucopy is a utility to modify database structures based on the"
  ? "structure of a set of databases in another directory"
  ?
  ? 'Source is the directory which contains the "mother" databases.'
  ? "destination is the directory where to create or modify the databases."
  ? "If destination is not specified then the current directory is used."
  ? "extension specifies the dbase file extension. If not specified,"
  ? "the default .DBF will be used. (include the point with the extension)"
  ?
  ? "Note: STRUCOPY checks for identical structures, to avoid unecessary"
  ? "      structure modifications. This version supports .dbt files"
  ?
return

Function DispBusy
  Local saverow,savecol
  Static Counter:=0
  ++RecCounter
  if ++Counter>=200
    Counter=0
    saverow=row()
    savecol=Col()
    ?? str(Int(RecCounter/NumOfRec*100),3,0)+"%"
    setpos(Saverow,savecol)
  endif
return (.T.)